@@ -1,6 +1,5 @@
 #!/usr/local/bin/perl -w
-use 5.008;
-use strict;
+use 5.014;
 use warnings;
 
 use Module::Build;
@@ -9,13 +8,13 @@ Module::Build->new(
     module_name       => 'Finance::YahooJPN::Quote',
     dist_version_from => 'Quote.pm',
     dist_author       => 'Masanori HATA <http://www.mihr.net> (Saitama, JAPAN)',
-    dist_abstract     => 'to get a quote in Japanese stock market',
+    dist_abstract     => 'For fetching histrical stock quotes in Japan from Yahoo! Japan Finance.',
     license           => 'perl',
     requires          => {
-        'perl'         => '5.8.0',
+        'perl'         => '5.14.0',
     },
     recommends        => {
-        'perl'         => '5.10.0',
+        'perl'         => '5.14.2',
     },
     pm_files          => {
         'Quote.pm'   => 'lib/Finance/YahooJPN/Quote.pm',
